@@ -117,15 +117,17 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "xf86Aiptek.h"
 #include <errno.h>
 #include <string.h>
 #include <math.h>
 
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+#include <X11/Xatom.h>
+#include <xserver-properties.h>
+#endif
+
+
 static const char identification[] = "$Identification: 0 $";
 static InputDriverPtr aiptekDrv;
 static int debug_level = INI_DEBUG_LEVEL;
@@ -141,25 +143,6 @@ _X_EXPORT InputDriverRec AIPTEK =
     0                   /* ref count */
 };
 
-/*
- * Function/Macro keys variables.
- *
- * This is a list of X keystrokes the macro keys can send.
- */
-static KeySym aiptek_map[] =
-{
-    /* 0x00 .. 0x07 */
-    NoSymbol,NoSymbol,NoSymbol,NoSymbol,NoSymbol,NoSymbol,NoSymbol,NoSymbol,
-    /* 0x08 .. 0x0f */
-    XK_F1,  XK_F2,  XK_F3,  XK_F4,  XK_F5,  XK_F6,  XK_F7,  XK_F8,
-    /* 0x10 .. 0x17 */
-    XK_F9,  XK_F10, XK_F11, XK_F12, XK_F13, XK_F14, XK_F15, XK_F16,
-    /* 0x18 .. 0x1f */
-    XK_F17, XK_F18, XK_F19, XK_F20, XK_F21, XK_F22, XK_F23, XK_F24,
-    /* 0x20 .. 0x27 */
-    XK_F25, XK_F26, XK_F27, XK_F28, XK_F29, XK_F30, XK_F31, XK_F32
-};
-
 /*
  * This is the map of Linux Event Input system keystrokes sent for
  * the macro keys. There are discrepancies in the mappings, so for example,
@@ -181,12 +164,33 @@ static int linux_inputDevice_keyMap[] =
     KEY_OPEN, KEY_PASTE
 };
 
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 7
+/*
+ * Function/Macro keys variables.
+ *
+ * This is a list of X keystrokes the macro keys can send.
+ */
+static KeySym aiptek_map[256] =
+{
+    /* 0x00 .. 0x07 */
+    NoSymbol,NoSymbol,NoSymbol,NoSymbol,NoSymbol,NoSymbol,NoSymbol,NoSymbol,
+    /* 0x08 .. 0x0f */
+    XK_F1,  XK_F2,  XK_F3,  XK_F4,  XK_F5,  XK_F6,  XK_F7,  XK_F8,
+    /* 0x10 .. 0x17 */
+    XK_F9,  XK_F10, XK_F11, XK_F12, XK_F13, XK_F14, XK_F15, XK_F16,
+    /* 0x18 .. 0x1f */
+    XK_F17, XK_F18, XK_F19, XK_F20, XK_F21, XK_F22, XK_F23, XK_F24,
+    /* 0x20 .. 0x27 */
+    XK_F25, XK_F26, XK_F27, XK_F28, XK_F29, XK_F30, XK_F31, XK_F32
+};
+
 /* minKeyCode = 8 because this is the min legal key code */
 static KeySymsRec keysyms =
 {
   /* map        minKeyCode  maxKC   width */
-  aiptek_map,   8,          0x27,   1
+  aiptek_map,   8,          0xff,   1
 };
+#endif
 
 static const char *default_options[] =
 {
@@ -444,26 +448,26 @@ xf86AiptekSendEvents(LocalDevicePtr local, int r_z)
                  sizeof(linux_inputDevice_keyMap[0]);
              ++i)
         {
-            if (linux_inputDevice_keyMap[0]==common->currentValues.macroKey)
+            if (linux_inputDevice_keyMap[i]==common->currentValues.macroKey)
             {
+                /* First available Keycode begins at 8 => macro+8.
+                 * It's pervasive throughout the Xinput drivers, and
+                 * no, I don't know why they purposively waste the first 8
+                 * positions of the KeySym map...
+                 */
+
+                /* Keyboard 'make' (press) event */
+                xf86PostKeyEvent(local->dev, i+8, TRUE,
+                                 bAbsolute, 0, 5,
+                                 x, y, common->currentValues.button, xTilt, yTilt);
+                /* Keyboard 'break' (depress) event */
+                xf86PostKeyEvent(local->dev, i+8, FALSE,
+                                 bAbsolute, 0, 5,
+                                 x, y, common->currentValues.button, xTilt, yTilt);
                 break;
             }
         }
 
-        /* First available Keycode begins at 8 => macro+7.
-         * It's pervasive throughout the Xinput drivers, and
-         * no, I don't know why they purposively waste the first 8
-         * positions of the KeySym map...
-         */
-
-        /* Keyboard 'make' (press) event */
-        xf86PostKeyEvent(local->dev, i+7, TRUE,
-                         bAbsolute, 0, 5,
-                         x, y, common->currentValues.button, xTilt, yTilt);
-        /* Keyboard 'break' (depress) event */
-        xf86PostKeyEvent(local->dev, i+7, FALSE,
-                         bAbsolute, 0, 5,
-                         x, y, common->currentValues.button, xTilt, yTilt);
     }
 
     /* As the coordinates are ready, we can send events to X */
@@ -858,7 +862,7 @@ xf86AiptekHIDReadInput(LocalDevicePtr local)
                     default:
                     {
                         ++eventsInMessage;
-                        common->currentValues.macroKey = event->value;
+                        common->currentValues.macroKey = event->code;
                     }
                     break;
                 }
@@ -1499,6 +1503,9 @@ xf86AiptekOpenDevice(DeviceIntPtr pDriver)
      */
     InitValuatorAxisStruct(pDriver,                 /* X resolution */
                0,                                   /* axis_id */
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+               XIGetKnownProperty(AXIS_LABEL_PROP_ABS_X),
+#endif
                0,                                   /* min value */
                device->xBottom - device->xTop,      /* max value */
                LPI2CPM(375),                        /* resolution */
@@ -1507,6 +1514,9 @@ xf86AiptekOpenDevice(DeviceIntPtr pDriver)
 
     InitValuatorAxisStruct(pDriver,                 /* Y Resolution */
                1,                                   /* axis_id */
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+               XIGetKnownProperty(AXIS_LABEL_PROP_ABS_Y),
+#endif
                0,                                   /* min value */
                device->yBottom - device->yTop,      /* max value */
                LPI2CPM(375),                        /* resolution */
@@ -1515,6 +1525,9 @@ xf86AiptekOpenDevice(DeviceIntPtr pDriver)
 
     InitValuatorAxisStruct(pDriver,                 /* Pressure */
                2,                                   /* axis_id */
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+               XIGetKnownProperty(AXIS_LABEL_PROP_ABS_PRESSURE),
+#endif
                0,                                   /* min value */
                511,                                 /* max value */
                512,                                 /* resolution */
@@ -1523,6 +1536,9 @@ xf86AiptekOpenDevice(DeviceIntPtr pDriver)
 
     InitValuatorAxisStruct(pDriver,                 /* xTilt */
                3,                                   /* axis id */
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+               XIGetKnownProperty(AXIS_LABEL_PROP_ABS_TILT_X),
+#endif
                -128,                                /* min value */
                127,                                 /* max value */
                256,                                 /* resolution */
@@ -1531,6 +1547,9 @@ xf86AiptekOpenDevice(DeviceIntPtr pDriver)
 
     InitValuatorAxisStruct(pDriver,                 /* yTilt */
                4,                                   /* axis_id */
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+               XIGetKnownProperty(AXIS_LABEL_PROP_ABS_TILT_Y),
+#endif
                -128,                                /* min value */
                127,                                 /* max value */
                256,                                 /* resolution */
@@ -1545,6 +1564,23 @@ xf86AiptekOpenDevice(DeviceIntPtr pDriver)
     return (local->fd != -1);
 }
 
+/*
+ * xf86AiptekBellCallback
+ */
+static void
+xf86AiptekBellCallback(int pct, DeviceIntPtr di, pointer ctrl, int x)
+{
+}
+
+/*
+ * xf86AiptekKbdCtrlCallback
+ */
+static void
+xf86AiptekKbdCtrlCallback(DeviceIntPtr di, KeybdCtrl* ctrl)
+{
+}
+
+
 /*
  * xf86AiptekProc
  *
@@ -1554,11 +1590,15 @@ static int
 xf86AiptekProc(DeviceIntPtr pAiptek, int requestCode)
 {
     CARD8           map[512+1];
-    int             numAxes;
-    int             numButtons;
+    int             numAxes = 5; /* X, Y, Z, xTilt, yTilt */
+    int             numButtons = 5;
     int             loop;
     LocalDevicePtr  local  = (LocalDevicePtr)pAiptek->public.devicePrivate;
     AiptekDevicePtr device = (AiptekDevicePtr)PRIVATE(pAiptek);
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+    Atom            btn_labels[numAxes];
+    Atom            axes_labels[numButtons];
+#endif
 
     DBG(2, ErrorF("xf86AiptekProc() type=%s flags=%d request=%d\n",
               (DEVICE_ID(device->flags) == STYLUS_ID) ? "stylus" :
@@ -1570,15 +1610,27 @@ xf86AiptekProc(DeviceIntPtr pAiptek, int requestCode)
         case DEVICE_INIT:
         {
             DBG(1, ErrorF("xf86AiptekProc request=INIT\n"));
-            numAxes    = 5;            /* X, Y, Z, xTilt, yTilt */
-            numButtons = 5;
 
             for(loop=1; loop<=numButtons; ++loop)
             {
                 map[loop] = loop;
             }
 
-            if (InitButtonClassDeviceStruct(pAiptek,numButtons,map) == FALSE)
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+            btn_labels[0] = XIGetKnownProperty(BTN_LABEL_PROP_BTN_LEFT);
+            btn_labels[1] = XIGetKnownProperty(BTN_LABEL_PROP_BTN_MIDDLE);
+            btn_labels[2] = XIGetKnownProperty(BTN_LABEL_PROP_BTN_RIGHT);
+            btn_labels[3] = XIGetKnownProperty(BTN_LABEL_PROP_BTN_WHEEL_UP);
+            btn_labels[4] = XIGetKnownProperty(BTN_LABEL_PROP_BTN_WHEEL_DOWN);
+
+            memset(axes_labels, 0, sizeof(axes_labels));
+#endif
+
+            if (InitButtonClassDeviceStruct(pAiptek,numButtons,
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+                        btn_labels,
+#endif
+                        map) == FALSE)
             {
                 ErrorF("Unable to init Button Class Device\n");
                 return !Success;
@@ -1603,15 +1655,33 @@ xf86AiptekProc(DeviceIntPtr pAiptek, int requestCode)
                 return !Success;
             }
 
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 7
             if (InitKeyClassDeviceStruct(pAiptek, &keysyms, NULL) ==FALSE)
             {
                 ErrorF("Unable to init Key Class Device\n");
                 return !Success;
             }
 
+            if(InitKbdFeedbackClassDeviceStruct(pAiptek,
+                xf86AiptekBellCallback,
+                xf86AiptekKbdCtrlCallback) == FALSE)
+            {
+                ErrorF("Unable to init Keyboard Feedback Class Device\n");
+                return !Success;
+            }
+
+#endif
+
+            /* we don't label the axes here, done later in
+             * xf86AiptedOpenDevice */
             if (InitValuatorClassDeviceStruct(pAiptek,
                    numAxes,
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) >= 7
+                   axes_labels,
+#endif
+#if GET_ABI_MAJOR(ABI_XINPUT_VERSION) < 3
                    xf86GetMotionEvents,
+#endif
                    local->history_size,
                    ((device->flags & ABSOLUTE_FLAG) 
                         ? Absolute : Relative) | OutOfProximity ) == FALSE)
@@ -1999,6 +2069,7 @@ xf86AiptekUninit(InputDriverPtr    drv,
         xfree(device->common);
     }
     xfree (device);
+    local->private = NULL;
     xf86DeleteInput(local, 0);
 }
 
@@ -2189,7 +2260,7 @@ xf86AiptekInit(InputDriverPtr    drv,
     xf86Msg(X_CONFIG, "%s is in %s mode\n", local->name,
         (device->flags & ABSOLUTE_FLAG) ? "absolute" : "relative");
 
-#ifdef LINUX_INPUT
+#ifdef HAVE_LINUX_INPUT_H
     /* The define-name is accurate; the XFree86 keyword is not. We are
      * reading from a Linux kernel "Input" device. The Input device
      * layer generally supports mice, joysticks, and keyboards. As
